@@ -7,14 +7,15 @@
  *   Copyright (C) 2004 Tony Finch <dot@dotat.at>
  *
  *  It is part of adns, which is
- *    Copyright (C) 1997-2000,2003,2006  Ian Jackson
+ *    Copyright (C) 1997-2000,2003,2006,2014-2016  Ian Jackson
+ *    Copyright (C) 2014  Mark Wooding
  *    Copyright (C) 1999-2000,2003,2006  Tony Finch
  *    Copyright (C) 1991 Massachusetts Institute of Technology
  *  (See the file INSTALL for full details.)
  *  
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
+ *  the Free Software Foundation; either version 3, or (at your option)
  *  any later version.
  *  
  *  This program is distributed in the hope that it will be useful,
@@ -23,15 +24,11 @@
  *  GNU General Public License for more details.
  *  
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software Foundation,
- *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation.
  *
  *  This file is by Tony Finch, based on adnslogres.c.
  */
 
-static const char * const cvsid =
-	"$Id: adnsheloex.c,v 1.3 2006/05/09 19:37:58 ian Exp $";
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
