@@ -4,14 +4,15 @@
  */
 /*
  *  This file is part of adns, which is
- *    Copyright (C) 1997-2000,2003,2006  Ian Jackson
+ *    Copyright (C) 1997-2000,2003,2006,2014-2016  Ian Jackson
+ *    Copyright (C) 2014  Mark Wooding
  *    Copyright (C) 1999-2000,2003,2006  Tony Finch
  *    Copyright (C) 1991 Massachusetts Institute of Technology
  *  (See the file INSTALL for full details.)
  *  
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
+ *  the Free Software Foundation; either version 3, or (at your option)
  *  any later version.
  *  
  *  This program is distributed in the hope that it will be useful,
@@ -20,8 +21,7 @@
  *  GNU General Public License for more details.
  *  
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software Foundation,
- *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
+ *  along with this program; if not, write to the Free Software Foundation.
  */
 
 #include <stdio.h>
@@ -32,11 +32,15 @@
 #include <string.h>
 #include <errno.h>
 
+#define ADNS_FEATURE_MANYAF
 #include "config.h"
 #include "adns.h"
 
 #ifdef ADNS_REGRESS_TEST
 # include "hredirect.h"
+# include "harness.h"
+# undef exit
+# define exit Texit
 #endif
 
 struct myctx {
@@ -101,6 +105,7 @@ static void usageerr(const char *why) {
 	  "initflags:   p  use poll(2) instead of select(2)\n"
 	  "             s  use adns_wait with specified query, instead of 0\n"
 	  "queryflags:  a  print status abbrevs instead of strings\n"
+	  "typenum:      may be 0x<hex>|<dec>, or 0x<hex> or <dec>\n"
 	  "exit status:  0 ok (though some queries may have failed)\n"
 	  "              1 used by test harness to indicate test failed\n"
 	  "              2 unable to submit or init or some such\n"
@@ -206,12 +211,19 @@ int main(int argc, char *const *argv) {
       if (ch==',') tc++;
     types_a= malloc(sizeof(*types_a)*(tc+1));
     if (!types_a) { perror("malloc types"); quitnow(3); }
-    for (cp= argv[1]+1, ti=0; ti<tc; ti++) {
-      types_a[ti]= strtoul(cp,&cp,10);
-      if ((ch= *cp)) {
-	if (ch != ',') usageerr("unexpected char (not comma) in or between types");
+    for (cp= argv[1]+1, ti=0; ti<tc; ) {
+      types_a[ti]= 0;
+      for (;;) {
+	types_a[ti] |= strtoul(cp,&cp,0);
+	ch= *cp;
+	if (!ch) break;
 	cp++;
+	if (ch=='|') continue;
+	if (ch==',') break;
+	usageerr("unexpected char (not comma) in or between types");
       }
+      ti++;
+      if (!ch) break;
     }
     types_a[ti]= adns_r_none;
     types= types_a;
@@ -251,7 +263,8 @@ int main(int argc, char *const *argv) {
       mc->doneyet= 0;
       mc->fdom= fdomlist[qi];
 
-      fprintf(stdout,"%s flags %d type %d",domain,qflags,types[ti]);
+      fprintf(stdout,"%s flags %d type %d",
+	      domain,qflags,types[ti]&adns_rrt_reprmask);
       r= adns_submit(ads,domain,types[ti],qflags,mc,&mc->qu);
       if (r == ENOSYS) {
 	fprintf(stdout," not implemented\n");
